import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Scanner;

import sd.constants.Constants;
import sd.data.*;
import sd.discovery.SubDisc;
import sd.discovery.Subgroup;

public class Test {
	public static void main(String[] args) {
		try {
			readInputString(args);
			String file = Constants.FILEIN;
			String fileOut = Constants.FILEOUT;
			int numNumeric = Constants.NUMERIC;
			int numOrdinal = Constants.ORDINAL;
			int numCategoric = Constants.CATEGORIC;
			int numNumericTarget = Constants.NUMTARGET;
			int numOrdinalTarget = Constants.ORDTARGET;
			int numCategoricTarget = Constants.CATTARGET;
			int method = Constants.METHOD;
			int measure = Constants.MEASURE;
			int mode = Constants.MODE;
			int option = Constants.OPTION;
			
			Database db = readData(file, numNumeric, numOrdinal, numCategoric, numNumericTarget, numOrdinalTarget, numCategoricTarget);
			long start = System.currentTimeMillis();
			Subgroup[] result = SubDisc.find(db, method, measure, mode, option);
			long end = System.currentTimeMillis();
			BufferedWriter writer = new BufferedWriter(new FileWriter(new File(fileOut)));
			for (Subgroup sub : result) {
				writer.write(sub.toString());
				writer.newLine();
			}
			writer.write(Long.toString(end - start));
			writer.newLine();
			writer.flush();
			writer.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public static Database readData(String file, int numNumeric, int numOrdinal, int numCategoric, int numNumericTarget, int numOrdinalTarget, int numCategoricTarget) 
			throws IOException, IllegalArgumentException {
		Scanner scan = null;
		try {
			scan = new Scanner(new File(file));
			
			ArrayList<DataPoint> points = new ArrayList<DataPoint>();
			String s;
			String[] vals;
			double[] numeric;
			int[] ordinal;
			String[] categoric;
			double[] numericTarget;
			int[] ordinalTarget;
			String[] categoricTarget;
			DataPoint p = null;
			int n = numNumeric + numOrdinal + numCategoric + numNumericTarget + numOrdinalTarget + numCategoricTarget;
			while (scan.hasNext()) {
				s = scan.nextLine();
				vals = s.split(",");
				if (vals.length != n) {
					System.out.println(s);
					throw new IllegalArgumentException("missing data!");
				}
				
				numeric = new double[numNumeric];
				ordinal = new int[numOrdinal];
				categoric = new String[numCategoric];
				numericTarget = new double[numNumericTarget];
				ordinalTarget = new int[numOrdinalTarget];
				categoricTarget = new String[numCategoricTarget];
				
				for (int i = 0; i < numNumeric; i++) {
					numeric[i] = Double.parseDouble(vals[i]);
				}
				
				for (int i = 0; i < numOrdinal; i++) {
					ordinal[i] = Integer.parseInt(vals[i + numNumeric]);
				}
				
				for (int i = 0; i < numCategoric; i++) {
					categoric[i] = vals[i + numNumeric + numOrdinal];
				}
				
				for (int i = 0; i < numNumericTarget; i++) {
					numericTarget[i] = Double.parseDouble(vals[i + numNumeric + numOrdinal + numCategoric]);
				}
				
				for (int i = 0; i < numOrdinalTarget; i++) {
					ordinalTarget[i] = Integer.parseInt(vals[i + numNumeric + numOrdinal + numCategoric + numNumericTarget]);
				}
				
				for (int i = 0; i < numCategoricTarget; i++) {
					categoricTarget[i] = vals[i + numNumeric + numOrdinal + numCategoric + numNumericTarget + numOrdinalTarget];
				}
				
				p = new DataPoint(numeric, ordinal, categoric, numericTarget, ordinalTarget, categoricTarget);
				points.add(p);
			}
			
			Database db = new Database(points);
			db.setNumNumeric(numNumeric);
			db.setNumOrdinal(numOrdinal);
			db.setNumCategoric(numCategoric);
			db.setNumNumericTarget(numNumericTarget);
			db.setNumOrdinalTarget(numOrdinalTarget);
			db.createOrdinalTargetCount();
			db.createCategoricTargetCount();
			db.createIndividualCategoricTargetCount();
			db.createNumericTargetMax();
			return db;
		} catch (IOException e) {
			throw e;
		} finally {
			if (scan != null) {
				scan.close();
			}
		}
	}
	
	public static void readInputString(String[] args) throws Exception
	{	
		int i;
		int total = args.length - 1;
		
		// take FILEIN
		boolean found = false;
		for (i = 0; i < total; i++)
			if (args[i].equals("-FILEIN"))
			{
				Constants.FILEIN = args[i + 1];
				found = true;
				break;
			}
		if (found == false)
			throw new Exception("Missing -FILEIN");
		
		// take FILEOUT
		found = false;
		for (i = 0; i < total; i++)
			if (args[i].equals("-FILEOUT"))
			{
				Constants.FILEOUT = args[i + 1];
				found = true;
				break;
			}
		if (found == false)
			throw new Exception("Missing -FILEOUT");
		
		// take FILESCORE
		found = false;
		for (i = 0; i < total; i++)
			if (args[i].equals("-FILESCORE"))
			{
				Constants.FILESCORE = args[i + 1];
				found = true;
				break;
			}
		
		// take NUMERIC
		found = false;
		for (i = 0; i < total; i++)
			if (args[i].equals("-NUMERIC"))
			{
				Constants.NUMERIC = Integer.parseInt(args[i + 1]);
				found = true;
				break;
			}
		if (found == false)
			throw new Exception("Missing -NUMERIC");
		
		// take ORDINAL
		found = false;
		for (i = 0; i < total; i++)
			if (args[i].equals("-ORDINAL"))
			{
				Constants.ORDINAL = Integer.parseInt(args[i + 1]);
				found = true;
				break;
			}
		if (found == false)
			throw new Exception("Missing -ORDINAL");
		
		// take CATEGORIC
		found = false;
		for (i = 0; i < total; i++)
			if (args[i].equals("-CATEGORIC"))
			{
				Constants.CATEGORIC = Integer.parseInt(args[i + 1]);
				found = true;
				break;
			}
		if (found == false)
			throw new Exception("Missing -CATEGORIC");
		
		// take NUMTARGET
		found = false;
		for (i = 0; i < total; i++)
			if (args[i].equals("-NUMTARGET"))
			{
				Constants.NUMTARGET = Integer.parseInt(args[i + 1]);
				found = true;
				break;
			}
		if (found == false)
			throw new Exception("Missing -NUMTARGET");
		
		// take ORDTARGET
		found = false;
		for (i = 0; i < total; i++)
			if (args[i].equals("-ORDTARGET"))
			{
				Constants.ORDTARGET = Integer.parseInt(args[i + 1]);
				found = true;
				break;
			}
		if (found == false)
			throw new Exception("Missing -ORDTARGET");
		
		// take CATTARGET
		found = false;
		for (i = 0; i < total; i++)
			if (args[i].equals("-CATTARGET"))
			{
				Constants.CATTARGET = Integer.parseInt(args[i + 1]);
				found = true;
				break;
			}
		if (found == false)
			throw new Exception("Missing -CATTARGET");
		
		// take MEASURE
		found = false;
		for (i = 0; i < total; i++)
			if (args[i].equals("-MEASURE"))
			{
				Constants.MEASURE = Integer.parseInt(args[i + 1]);
				found = true;
				break;
			}
		if (found == false)
			throw new Exception("Missing -MEASURE");
		
		// take BETA
		found = false;
		for (i = 0; i < total; i++)
			if (args[i].equals("-BETA"))
			{
				Constants.BETA = Integer.parseInt(args[i + 1]);
				found = true;
				break;
			}
		
		// take SAMPLING
		found = false;
		for (i = 0; i < total; i++)
			if (args[i].equals("-SAMPLING"))
			{
				Constants.SAMPLING = Boolean.parseBoolean(args[i + 1]);
				found = true;
				break;
			}
		
		// take EPSILON
		found = false;
		for (i = 0; i < total; i++)
			if (args[i].equals("-EPSILON"))
			{
				Constants.EPSILON = Double.parseDouble(args[i + 1]);
				found = true;
				break;
			}
	} // end method
}