package sd.computer;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import sd.constants.Constants;
import sd.data.Database;

public class HellingerDiv {
	public static double score(Database db, List<Integer> pids) {
		HashMap<String[], Integer> stats = new HashMap<String[], Integer>();
		for (Integer id : pids) {
			String[] key = db.getPoints().get(id).getCategoricTarget();
			if (stats.containsKey(key)) {
				int a = stats.get(key) + 1;
				stats.put(key, a);
			} else {
				stats.put(key, 1);
			}
		}
		
		Map<String[], Integer> pops = db.getCategoricTargetCount();
		double sum = 0;
		for (String[] key : pops.keySet()) {
			double a = pops.get(key) * 1.0 / db.getPoints().size();
			double b = 0;
			if (stats.containsKey(key)) {
				b = stats.get(key) * 1.0 / pids.size();
			}
			
			double v = Math.sqrt(b) - Math.sqrt(a);
			sum += v * v; 
		}
		
		return sum;
	}
	
	public static double score(Database db, List<Integer> a, List<Integer> b) {
		HashMap<String[], Integer> statsA = KLDiv.getMap(db, a);
		HashMap<String[], Integer> statsB = KLDiv.getMap(db, b);
		
		return HDScore(statsA, a.size(), statsB, b.size());
	}
	
	private static double HDScore(HashMap<String[], Integer> statsA, int sizeA, HashMap<String[], Integer> statsB, int sizeB) {
		double sum = 0;
		for (String[] key : statsA.keySet()) {
			double a = statsA.get(key) * 1.0 / sizeA;
			if (a == 0) {
				throw new ArithmeticException("invalid frequency computation!");
			}
			
			double b = 0;
			if (statsB.containsKey(key)) {
				b = statsB.get(key) * 1.0 / sizeB;
			}
			
			double v = Math.sqrt(b) - Math.sqrt(a);
			sum += v * v; 
		}
		
		for (String[] key : statsB.keySet()) {
			if (!statsA.containsKey(key)) {
				sum += statsB.get(key) * 1.0 / sizeB;
			}
		}
		
		return sum;
	}
	
	public static double totalScore(Database db, List<Integer> pids, int option) {
		double t = score(db, pids);
		
		if (option == Constants.FREQUENCY) { // frequency
			t = pids.size() * t / db.getPoints().size();
		} else { // entropy of frequency
			if (pids.size() < db.getPoints().size()) {
				t = t * Math.sqrt((- (pids.size() * 1.0 / db.getPoints().size()) * Math.log(pids.size() * 1.0 / db.getPoints().size()) / Math.log(2) 
						- ((db.getPoints().size() - pids.size()) * 1.0 / db.getPoints().size()) * Math.log((db.getPoints().size() - pids.size()) * 1.0 / db.getPoints().size()) / Math.log(2)));
			} else {
				t = 0;
			}
		}
		
		return t;
	}
}