package sd.computer;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

import sd.constants.Constants;
import sd.data.*;
import sd.score.Score;

public class QuadraticMeasure {
	public static double firstTerm(Database db, List<Integer> pids) {
		return score(db, pids, pids);
	}
	
	public static double secondTerm(Database db, List<Integer> pids) {
		if (Constants.SAMPLING && Constants.SUBSAMPLE != null) {
			return score(db, pids, Constants.SUBSAMPLE);
		} else {
			int numPoints = db.getPoints().size();
			ArrayList<Integer> ids = new ArrayList<Integer>();
			for (int i = 0; i < numPoints; i++) {
				ids.add(i);
			}
			return score(db, pids, ids);
		}
	}
	
	public static double score(Database db, List<Integer> p1, List<Integer> p2) {
		double t = 0;
		int s1 = p1.size();
		int s2 = p2.size();
		int numOrdinalTarget = db.getNumOrdinalTarget();
		int numNumericTarget = db.getNumNumericTarget();
		Map<Integer, Integer>[] map = db.getOrdinalTargetCount();
		double[] max = db.getNumericTargetMax();
		ArrayList<DataPoint> points = db.getPoints();
		DataPoint o1, o2;
		int v1, v2;
		
		for (int i = 0; i < s1; i++) {
			o1 = points.get(p1.get(i));
			
			for (int j = 0; j < s2; j++) {
				o2 = points.get(p2.get(j));
				
				double prod = 1;
				for (int k = 0; k < numOrdinalTarget; k++) {
					v1 = o1.getOrdinalTarget()[k];
					v2 = o2.getOrdinalTarget()[k];
					prod = prod * map[k].get(v1 > v2 ? v1 : v2);
				}
				
				for (int k = 0; k < numNumericTarget; k++) {
					prod = prod * (max[k] - Math.max(o1.getNumericTarget()[k], o2.getNumericTarget()[k]));
				}
				
				t += prod;
			}
		}
		
		return t;
	}
	
	public static double totalScore(Score score, double e, int mode, int option) {
		double t = 0;
		if (mode == Constants.SD) { // subgroup discovery
			t = score.first / (score.s * score.s) - 2 * score.second / (score.s * score.m) + e / (score.m * score.m);
		} else { // exceptional model mining
			t = score.first / (score.s * score.s);
			if (score.s < score.m) {
				t = t - 2 * (score.second - score.first) / (score.s * (score.m - score.s))
						+ (e + score.first - 2 * score.second) / ((score.m - score.s) * (score.m - score.s));
			}
		}
		
		if (option == Constants.FREQUENCY) { // frequency
			t = score.s * t / score.m;
		} else { // entropy of frequency
			if (score.s < score.m) {
				t = t * Math.sqrt((- (score.s * 1.0 / score.m) * Math.log(score.s * 1.0 / score.m) / Math.log(2) 
						- ((score.m - score.s) * 1.0 / score.m) * Math.log((score.m - score.s) * 1.0 / score.m) / Math.log(2)));
			} else {
				t = 0;
			}
		}
		
		return t;
	}
}