package sd.discovery;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.HashSet;
import java.util.PriorityQueue;
import java.util.Scanner;

import sd.computer.*;
import sd.constants.Constants;
import sd.data.*;
import sd.discretizer.Bin;
import sd.discretizer.Discretization;

public class SubDisc {
	private static final int MIN_COV = 10;
	private static final int MAX_DEPTH = 3;
	private static final int W = 100;
	private static final int K = 50;
	
	public static Subgroup[] find(Database db, int method, int measure, int mode, int option) throws Exception {
		int m = db.getPoints().size();
		ArrayList<Integer> pids = new ArrayList<Integer>();
		for (int i = 0; i < m; i++) {
			pids.add(i);
		}
		
		int mpos = 0;
		double mean = 0;
		double std = 0;
		if (measure == Constants.WRACC || measure == Constants.MEANSTD) {
			if (measure == Constants.WRACC) {
				mpos = Discretization.countPos(db, pids);
			} else {
				mean = MeanStd.mean(db, pids);
				std = MeanStd.std(db, pids, mean);
			}
		}
		
		double e = 0;
		if (measure == Constants.QR) {
			e = getQRScore(Constants.FILESCORE);
		}
		
		PriorityQueue<Subgroup> R = new PriorityQueue<Subgroup>();
		ArrayList<Subgroup> Beam = new ArrayList<Subgroup>();
		int depth = 1;
		int n = db.getNumNumeric() + db.getNumOrdinal() + db.getNumCategoric();
		
		while (depth <= MAX_DEPTH) {
			PriorityQueue<Subgroup> Cands = new PriorityQueue<Subgroup>();
			if (Beam.size() == 0) {
				for (int i = 0; i < n; i++) {
					ArrayList<Subgroup> tmp = Discretization.generateRefinements(db, pids, i, e, method, measure, mode, option, true);
					for (Subgroup sub : tmp) {
						if (Constants.SAMPLING) {
							sub.setQuality(db, e, m, mpos, mean, std, method, measure, mode, option);
						}
						
						if (sub.getPoints().size() > MIN_COV) {
							if (Cands.size() < W) {
								Cands.offer(sub);
							} else {
								if (sub.getQuality() > Cands.peek().getQuality()) {
									Cands.poll();
									Cands.offer(sub);
								}
							}
						}
					}
				}
			} else {
				for (Subgroup sub : Beam) {
					int lastAttributeId = sub.getCons().get(sub.getCons().size() - 1).getId();
					for (int i = lastAttributeId + 1; i < n; i++) {
						ArrayList<Subgroup> tmp = null;
						if (i >= db.getNumNumeric() + db.getNumOrdinal()) {
							tmp = Discretization.generateRefinements(db, sub.getPoints(), i, e, method, measure, mode, option, false);
						} else {
							Subgroup s = michealRefinement(db, sub.getPoints(), i, e, m, mpos, mean, std, method, measure, mode, option);
							tmp = new ArrayList<Subgroup>();
							tmp.add(s);
						}
						
						for (Subgroup d : tmp) {
							Subgroup newSub = new Subgroup();
							newSub.getCons().addAll(sub.getCons());
							newSub.getCons().add(d.getCons().get(0));
							newSub.setPoints(d.getPoints());
							//if (Constants.SAMPLING) {
							//	newSub.setSubPoints(Discretization.subsample(newSub.getPoints(), Constants.EPSILON));
							//}
							if (newSub.getPoints().size() > MIN_COV) {
								newSub.setQuality(db, e, m, mpos, mean, std, method, measure, mode, option);
								
								if (Cands.size() < W) {
									Cands.offer(newSub);
								} else {
									if (newSub.getQuality() > Cands.peek().getQuality()) {
										Cands.poll();
										Cands.offer(newSub);
									}
								}
							}
						}
					}
				}
			}
			
			// update top K
			for (Subgroup sub : Cands) {
				if (R.size() < K) {
					R.offer(sub);
				} else {
					if (sub.getQuality() > R.peek().getQuality()) {
						R.poll();
						R.offer(sub);
					}
				}
			}
			
			Beam.clear();
			Beam.addAll(Cands);
			depth++;
		}
		
		Subgroup[] ret = new Subgroup[R.size()];
		ret = R.toArray(ret);
		Arrays.sort(ret, Collections.reverseOrder());
			
		return ret;
	}
	
	private static Subgroup michealRefinement(Database db, ArrayList<Integer> pids, int dim, double e, int m, int mpos,
											  double mean, double std, int method, int measure, int mode, int option) {
		HashSet<Double> set = new HashSet<Double>();
		for (Integer p : pids) {
			if (dim < db.getNumNumeric()) {
				set.add(db.getPoints().get(p).getNumeric()[dim]);
			} else {
				set.add((double)db.getPoints().get(p).getOrdinal()[dim - db.getNumNumeric()]);
			}
		}
		
		Double[] vals = new Double[set.size()];
		vals = set.toArray(vals);
		Arrays.sort(vals);
		
		double l = Double.NEGATIVE_INFINITY;
		double r = Double.POSITIVE_INFINITY;
		double phimax = phi(db, pids, dim, e, m, mpos, mean, std, method, measure, mode, option, l, r);
		double hmax = Double.NEGATIVE_INFINITY;
		double tmax = Double.NEGATIVE_INFINITY;
		double h;
		for (int i = 1; i < vals.length; i++) {
			h = phi(db, pids, dim, e, m, mpos, mean, std, method, measure, mode, option, vals[i - 1], Double.POSITIVE_INFINITY);
			
			if (h > hmax) {
				hmax = h;
				tmax = vals[i - 1];
			}
			
			double tmp = phi(db, pids, dim, e, m, mpos, mean, std, method, measure, mode, option, tmax, vals[i]);
			if (tmp > phimax) {
				l = tmax;
				r = vals[i];
				phimax = tmp;
			}
		}
		
		ArrayList<Integer> ids = new ArrayList<Integer>();
		Double min = Double.POSITIVE_INFINITY;
		Double max = Double.NEGATIVE_INFINITY;
		double val;
		for (Integer p : pids) {
			if (dim < db.getNumNumeric()) {
				val = db.getPoints().get(p).getNumeric()[dim];
			} else {
				val = db.getPoints().get(p).getOrdinal()[dim - db.getNumNumeric()];
			}
			
			if (val > l && val <= r) {
				ids.add(p);
			}
			
			if (val < min) {
				min = val;
			}
			
			if (val > max) {
				max = val;
			}
		}
		
		Subgroup sub = new Subgroup();
		sub.setPoints(ids);
		Attribute att = new Attribute(dim, min, max);
		sub.getCons().add(att);
		sub.setQuality(db, e, m, mpos, mean, std, method, measure, mode, option);
		return sub;
	}
	
	private static double phi(Database db, ArrayList<Integer> pids, int dim, double e, int m, int mpos,
			  double mean, double std, int method, int measure, int mode, int option, double l, double r) {
		ArrayList<Integer> ids = new ArrayList<Integer>();
		double val;
		for (Integer p : pids) {
			if (dim < db.getNumNumeric()) {
				val = db.getPoints().get(p).getNumeric()[dim];
			} else {
				val = db.getPoints().get(p).getOrdinal()[dim - db.getNumNumeric()];
			}
			
			if (val > l && val <= r) {
				ids.add(p);
			}
		}
		
		Bin tmp = new Bin(ids);
		tmp.setQuality(db, e, m, mpos, mean, std, measure, mode, option);
		return tmp.getQuality();
	}
	
	public static double getQRScore(String fileScore) throws Exception {
		Scanner scan = new Scanner(new File(fileScore));
		double t = Double.parseDouble(scan.nextLine());
		scan.close();
		return t;
	}
}