package sd.discovery;

import java.util.ArrayList;

import sd.computer.HellingerDiv;
import sd.computer.KLDiv;
import sd.computer.MeanStd;
import sd.computer.QuadraticMeasure;
import sd.computer.WRAcc;
import sd.constants.Constants;
import sd.data.Database;
import sd.discretizer.Bin;
import sd.discretizer.Discretization;
import sd.score.Score;

public class Subgroup implements Comparable<Subgroup> {
	private ArrayList<Attribute> cons;
	private ArrayList<Integer> pids;
	private ArrayList<Integer> subpids;
	private double quality = -10;
	
	public Subgroup() {
		cons = new ArrayList<Attribute>();
		pids = new ArrayList<Integer>();
	}
	
	public ArrayList<Attribute> getCons() {
		return cons;
	}
	public void setCons(ArrayList<Attribute> cons) {
		this.cons = cons;
	}
	
	public ArrayList<Integer> getPoints() {
		return pids;
	}
	public void setPoints(ArrayList<Integer> pids) {
		this.pids = pids;
	}
	
	public ArrayList<Integer> getSubPoints() {
		return subpids;
	}
	public void setSubPoints(ArrayList<Integer> subpids) {
		this.subpids = subpids;
	}
	
	public double getQuality() {
		return quality;
	}
	public void setQuality(double quality) {
		this.quality = quality;
	}
	public void setQuality(Database db, double e, int m, int mpos, double mean, double std, int method, int measure, int mode, int option) {
		if (measure == Constants.QR) {
			this.setQuality(QuadraticMeasure.totalScore(Score.score(db, new Bin(this.getPoints())), e, mode, option));
		} else if (measure == Constants.KL || measure == Constants.KLIND) {
			this.setQuality(KLDiv.totalScore(db, this.getPoints(), method, option));
		} else if (measure == Constants.HELL) {
			this.setQuality(HellingerDiv.totalScore(db, this.getPoints(), option));
		} else if (measure == Constants.WRACC) {
			this.setQuality(WRAcc.score(this.getPoints().size(), Discretization.countPos(db, this.getPoints()), m, mpos, option));
		} else {
			this.setQuality(MeanStd.score(this.getPoints().size(), MeanStd.mean(db, this.getPoints()), mean, std));
		}
	}
	
	@Override
	public String toString() {
		StringBuilder s = new StringBuilder();
		for (Attribute att : cons) {
			s.append(att.toString());
			s.append("+++");
		}
		s.append(Integer.toString(pids.size()) + ";");
		s.append(Double.toString(quality));
		
		return s.toString();
	}

	@Override
	public int compareTo(Subgroup o) {
		if (this.getQuality() > o.getQuality()) {
			return 1;
		} else if (this.getQuality() == o.getQuality()) {
			return 0;
		}
		
		return -1;
	}
}