package sd.discretizer;

import java.util.ArrayList;

import sd.computer.HellingerDiv;
import sd.computer.KLDiv;
import sd.computer.MeanStd;
import sd.computer.QuadraticMeasure;
import sd.computer.WRAcc;
import sd.constants.Constants;
import sd.data.Database;
import sd.score.Score;

public class Bin {
	private ArrayList<Integer> pids;
	private ArrayList<Integer> subpids;
	private double quality;
	private int minMicroBinID;
	private int maxMicroBinID;
	private int numMDHs;
	
	public Bin(ArrayList<Integer> pids) {
		setPoints(pids);
	}
	
	public ArrayList<Integer> getPoints() {
		return pids;
	}
	public void setPoints(ArrayList<Integer> pids) {
		this.pids = pids;
	}
	
	public ArrayList<Integer> getSubPoints() {
		return subpids;
	}
	public void setSubPoints(ArrayList<Integer> subpids) {
		this.subpids = subpids;
	}
	
	public double getQuality() {
		return quality;
	}
	public void setQuality(double quality) {
		this.quality = quality;
	}
	public void setQuality(Database db, double e, int m, int mpos, double mean, double std, int measure, int mode, int option) {
		if (measure == Constants.QR) {
			this.setQuality(QuadraticMeasure.totalScore(Score.score(db, new Bin(this.getPoints())), e, mode, option));
		} else if (measure == Constants.KL || measure == Constants.KLIND) {
			this.setQuality(KLDiv.totalScore(db, this.getPoints(), measure, option));
		} else if (measure == Constants.HELL) {
			this.setQuality(HellingerDiv.totalScore(db, this.getPoints(), option));
		} else if (measure == Constants.WRACC) {
			this.setQuality(WRAcc.score(this.getPoints().size(), Discretization.countPos(db, this.getPoints()), m, mpos, option));
		} else {
			this.setQuality(MeanStd.score(this.getPoints().size(), MeanStd.mean(db, this.getPoints()), mean, std));
		}
	}
	
	public int getMinMicroBinID() {
		return minMicroBinID;
	}
	public void setMinMicroBinID(final int minMicroBinID) {
		this.minMicroBinID = minMicroBinID;
	}
	
	public int getMaxMicroBinID() {
		return maxMicroBinID;
	}
	public void setMaxMicroBinID(final int maxMicroBinID) {
		this.maxMicroBinID = maxMicroBinID;
	}
	
	public int getNumMDHs() {
		return numMDHs;
	}
	public void setNumMDHs(int numMDHs) {
		this.numMDHs = numMDHs;
	}
}